﻿/**
* Sample code for the GetSignatureImage Canada Post service.
* 
* The GetSignatureImage  service  returns a signature image captured at delivery 
* of the parcel if available. (The parcel is identified using a PIN only). Please note 
* the following:
*   - U.S.A. and international services do not support signature retrieval.
*   - Signature images are available for 45 days after the last scan.
*   - Recipients of parcels may request suppression of their signature image from view.
*
* This sample is configured to access the Developer Program sandbox environment. 
* Use your development key username and password for the web service credentials.
* 
**/

using System;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Configuration;
using System.IO;
using System.Security.Authentication;
using System.Net;

namespace GetSignatureImage
{
    class GetSignatureImage
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username and password are imported from the following file
            // CPCWS_SOAP_Tracking_DotNet_Samples\SOAP\tracking\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];

            TrackingPortTypeClient server = new TrackingPortTypeClient("TrackingPort", "https://ct.soa-gw.canadapost.ca/vis/soap/track");
            server.ClientCredentials.UserName.UserName = username;
            server.ClientCredentials.UserName.Password = password;

            // Create Request
            getsignatureimagerequest request = new getsignatureimagerequest();
            request.locale = "FR";
            request.pin = "1371134583769923";

            // Disable timestamp on request
            BindingElementCollection elements = server.Endpoint.Binding.CreateBindingElements();
            elements.Find<SecurityBindingElement>().IncludeTimestamp = false;
            server.Endpoint.Binding = new CustomBinding(elements);

            String responseAsString = "";

            try
            {
                // Execute Request
                getsignatureimageresponse response = server.GetSignatureImage(request);

                // Retrieve values from response object
                if (response.Item.GetType() == typeof(getsignatureimageresponseSignatureimage))
                {
                    getsignatureimageresponseSignatureimage signature = (getsignatureimageresponseSignatureimage)response.Item;
                    var fileName = signature.filename;
                    var encodedImage = signature.image;
                    responseAsString += "Base64 Encoded: " + encodedImage + "\r\n";
                    responseAsString += "File name: " + fileName + "\r\n";
                    responseAsString += "Mime type: " + signature.mimetype + "\r\n";
                    byte[] decodedImage = Convert.FromBase64String(encodedImage);
                    FileStream fileStream = File.Create(fileName);
                    fileStream.Write(decodedImage, 0, decodedImage.Length);
                    responseAsString += "Decoding to " + Directory.GetCurrentDirectory() + "\\" + fileName + "\r\n";
                    fileStream.Close();
                }
                else
                {
                    messages msgs = (messages)response.Item;
                    foreach (var item in msgs.message)
                    {
                        responseAsString += "Error Code: " + item.code + "\r\n";
                        responseAsString += "Error Msg: " + item.description + "\r\n";
                    }
                }
            }
            catch (FaultException faultEx)
            {
                // SOAP Fault
                responseAsString += "Fault Code: " + faultEx.Code.Name + "\r\n";
                responseAsString += "Fault String: " + faultEx.Message + "\r\n";
            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message + "\r\n";
            }

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);

        }
    }
}
